package adminkindboxreqhandler

import (
	"net/http"
	"strconv"

	param "git.gocasts.ir/ebhomengo/niki/param/admin/kind_box_req"
	errmsg "git.gocasts.ir/ebhomengo/niki/pkg/err_msg"
	httpmsg "git.gocasts.ir/ebhomengo/niki/pkg/http_msg"
	echo "github.com/labstack/echo/v4"
)

// Deliver godoc
// @Summary      Admin deliver a kindboxreq
// @Tags         KindBoxReq
// @Accept       json
// @Produce      json
// @Param 		 id path int true "KindBoxReq ID"
// @Success      200  {object}   param.DeliverKindBoxReqResponse
// @Failure      400  {string}  "Bad request"
// @Security 	 AuthBearerAdmin
// @Router       /admin/kindboxreqs/deliver-kind-box-req/{id} [patch]
func (h Handler) Deliver(c echo.Context) error {
	var req param.DeliverKindBoxReqRequest

	id, cErr := strconv.ParseUint(c.Param("id"), 10, 64)

	if cErr != nil {
		return c.JSON(http.StatusBadRequest, errmsg.ErrorMsgInvalidInput)
	}

	req.KindBoxReqID = uint(id)

	if fieldErrors, err := h.adminKindBoxReqVld.ValidateDeliver(req); err != nil {
		msg, code := httpmsg.Error(err)

		return c.JSON(code, echo.Map{
			"message": msg,
			"errors":  fieldErrors,
		})
	}

	resp, sErr := h.adminKindBoxReqSvc.Deliver(c.Request().Context(), req)

	if sErr != nil {
		msg, code := httpmsg.Error(sErr)

		return echo.NewHTTPError(code, msg)
	}

	return c.JSON(http.StatusOK, resp)
}
