package mysqladdress

import (
	"context"
	"database/sql"
	"errors"
	"time"

	"git.gocasts.ir/ebhomengo/niki/entity"
	errmsg "git.gocasts.ir/ebhomengo/niki/pkg/err_msg"
	richerror "git.gocasts.ir/ebhomengo/niki/pkg/rich_error"
	"git.gocasts.ir/ebhomengo/niki/repository/mysql"
)

// TODO - use this approach or return (bool,entity.Address,error).
func (d *DB) GetAddressByID(ctx context.Context, id uint) (*entity.Address, error) {
	const op = "mysqladdress.IsExistAddressByID"

	row := d.conn.Conn().QueryRowContext(ctx, `select * from addresses where id = ?`, id)

	address, err := scanAddress(row)
	if err != nil {
		sErr := sql.ErrNoRows
		//TODO-errorsas: second argument to errors.As should not be *error
		//nolint
		if errors.As(err, &sErr) {
			return nil, nil
		}

		// TODO - log unexpected error for better observability
		return nil, richerror.New(op).WithErr(err).
			WithMessage(errmsg.ErrorMsgCantScanQueryResult).WithKind(richerror.KindUnexpected)
	}

	return &address, nil
}

func scanAddress(scanner mysql.Scanner) (entity.Address, error) {
	var createdAt, updatedAt time.Time
	var address entity.Address

	err := scanner.Scan(&address.ID, &address.PostalCode, &address.Address, &address.Lat, &address.Lon,
		&address.IsMain, &address.CityID, &address.ProvinceID, &address.BenefactorID,
		&createdAt, &updatedAt)

	return address, err
}
