package benefactorkindboxreqvalidator

import (
	"errors"

	param "git.gocasts.ir/ebhomengo/niki/param/benefactor/kind_box_req"
	errmsg "git.gocasts.ir/ebhomengo/niki/pkg/err_msg"
	richerror "git.gocasts.ir/ebhomengo/niki/pkg/rich_error"
	validation "github.com/go-ozzo/ozzo-validation/v4"
)

func (v Validator) ValidateAddRequest(req param.KindBoxReqAddRequest) *ValidatorError {
	const op = "userkindboxreqvalidator.ValidateAddRequest"

	if err := validation.ValidateStruct(&req,

		validation.Field(&req.CountRequested, validation.Required, validation.Min(MinKindBoxReq), validation.Max(MaxKindBoxReq)),

		validation.Field(&req.BenefactorID,
			validation.Required,
			validation.By(v.doesBenefactorExist)),

		validation.Field(&req.TypeID,
			validation.Required,
			validation.By(v.doesTypeExist)),

		validation.Field(&req.DeliverAddressID,
			validation.Required,
			validation.By(v.doesAddressExist(req.BenefactorID))),

		validation.Field(&req.DeliverReferDate,
			validation.Required,
		),
	); err != nil {

		fieldErrors := make(map[string]string)

		var errV validation.Errors
		if errors.As(err, &errV) {
			for key, value := range errV {
				if value != nil {
					fieldErrors[key] = value.Error()
				}
			}
		}

		return &ValidatorError{
			Fields: fieldErrors,
			Err: richerror.New(op).
				WithMessage(errmsg.ErrorMsgInvalidInput).
				WithKind(richerror.KindInvalid).
				WithMeta(map[string]interface{}{"req": req}).
				WithErr(err),
		}

	}

	return nil
}
